@@ -1,6 +1,7 @@
 """archive_tools.py - Archive tool functions."""
 
 import os
+import re
 import zipfile
 import tarfile
 
@@ -12,6 +13,34 @@ from mcomix.archive import tar
 from mcomix.archive import sevenzip
 from mcomix.archive import lha
 
+def szip_available():
+    return sevenzip.SevenZipArchive.is_available()
+
+def rar_available():
+    return rar.RarArchive.is_available() or szip_available()
+
+def lha_available():
+    return lha.LhaArchive.is_available() or szip_available()
+
+def get_supported_archive_regex():
+    """ Returns a compiled regular expression that contains extensions
+    of all currently supported file types, based on available applications.
+    """
+    formats = list(constants.ZIP_FORMATS[1] + constants.TAR_FORMATS[1])
+
+    if szip_available():
+        formats.extend(constants.SZIP_FORMATS[1])
+
+    if rar_available():
+        formats.extend(constants.RAR_FORMATS[1])
+
+    if lha_available():
+        formats.extend(constants.LHA_FORMATS[1])
+
+    # Strip leading glob characters "*." from file extensions
+    formats = [format[2:] for format in formats]
+    return re.compile(r'\.(' + '|'.join(formats) + r')\s*$', re.I)
+
 def archive_mime_type(path):
     """Return the archive type of <path> or None for non-archives."""
     try:
@@ -28,15 +57,19 @@ def archive_mime_type(path):
             magic = fd.read(4)
             fd.close()
 
-            if tarfile.is_tarfile(path) and os.path.getsize(path) > 0:
+            try:
+                istarfile = tarfile.is_tarfile(path)
+            except IOError:
+                # Tarfile raises an error when accessing certain network shares
+                istarfile = False
 
+            if istarfile and os.path.getsize(path) > 0:
                 if magic.startswith('BZh'):
                     return constants.BZIP2
-
-                if magic.startswith('\037\213'):
+                elif magic.startswith('\037\213'):
                     return constants.GZIP
-
-                return constants.TAR
+                else:
+                    return constants.TAR
 
             if magic == 'Rar!':
                 return constants.RAR
