@@ -9,6 +9,7 @@ from mcomix.preferences import prefs
 from mcomix import constants
 from mcomix import portability
 from mcomix import keybindings
+from mcomix import openwith
 
 
 class EventHandler:
@@ -49,16 +50,23 @@ class EventHandler:
         """ Registers keyboard events and their default binings, and hooks
         them up with their respective callback functions. """
 
-        manager = keybindings.keybinding_manager()
+        manager = keybindings.keybinding_manager(self._window)
 
         # Navigation keys that work in addition to the accelerators in ui.py
         manager.register('previous page',
             ['KP_Page_Up', 'BackSpace', '<Mod1>Left'],
-            self._previous_page_with_protection)
+            self._window.previous_page)
         manager.register('next page',
             ['KP_Page_Down', '<Mod1>Right'],
             self._window.next_page)
 
+        manager.register('previous page ff',
+            ['<Shift>Page_Up', '<Shift>KP_Page_Up', '<Shift>BackSpace', '<Shift><Mod1>Left'],
+            self._window.previous_page_fast_forward)
+        manager.register('next page ff',
+            ['<Shift>Page_Down', '<Shift>KP_Page_Down', '<Shift><Mod1>Right'],
+            self._window.next_page_fast_forward)
+
         # Numpad (without numlock) aligns the image depending on the key.
         manager.register('scroll left bottom',
             ['KP_1'],
@@ -109,13 +117,11 @@ class EventHandler:
 
         # Zooming commands for manual zoom mode
         manager.register('zoom in',
-            ['plus', 'equal'],
+            ['equal'],
             self._window.actiongroup.get_action('zoom_in').activate)
-        manager.register('zoom out',
-            ['minus'],
-            self._window.actiongroup.get_action('zoom_out').activate)
+        # Zoom out is already defined as GTK menu hotkey
         manager.register('zoom original',
-            ['<Control>0', '<Control>KP_0'],
+            ['KP_0'],
             self._window.actiongroup.get_action('zoom_original').activate)
 
         # Arrow keys scroll the image
@@ -152,6 +158,11 @@ class EventHandler:
             ['Tab'],
             self._window.show_info_panel)
 
+        # Execute external command. Bind keys from 1 to 9 to commands 1 to 9.
+        for i in range(1, 10):
+            manager.register('execute command %d' % i, ['%d' % i],
+                             self._execute_command, args=[i - 1])
+
     def key_press_event(self, widget, event, *args):
         """Handle key press events on the main window."""
 
@@ -159,7 +170,7 @@ class EventHandler:
         self._scroll_protection = False
 
         # Dispatch keyboard input handling
-        manager = keybindings.keybinding_manager()
+        manager = keybindings.keybinding_manager(self._window)
         manager.execute((event.keyval, event.state))
 
         # ---------------------------------------------------------------
@@ -245,7 +256,9 @@ class EventHandler:
         elif event.button == 2:
             self._window.actiongroup.get_action('lens').set_active(True)
 
-        elif event.button == 3 and not event.state & gtk.gdk.MOD1_MASK:
+        elif (event.button == 3 and
+              not event.state & gtk.gdk.MOD1_MASK and
+              not event.state & gtk.gdk.SHIFT_MASK):
             self._window.cursor_handler.set_cursor_type(constants.NORMAL_CURSOR)
             self._window.popup.popup(None, None, None, event.button,
                 event.time)
@@ -263,7 +276,11 @@ class EventHandler:
             if event.x_root == self._pressed_pointer_pos_x and \
                 event.y_root == self._pressed_pointer_pos_y and \
                 not self._window.was_out_of_focus:
-                self._window.next_page()
+
+                if event.state & gtk.gdk.SHIFT_MASK:
+                    self._window.next_page_fast_forward()
+                else:
+                    self._window.next_page()
 
             else:
                 self._window.was_out_of_focus = False
@@ -271,8 +288,11 @@ class EventHandler:
         elif event.button == 2:
             self._window.actiongroup.get_action('lens').set_active(False)
 
-        elif event.button == 3 and event.state & gtk.gdk.MOD1_MASK:
-            self._previous_page_with_protection()
+        elif event.button == 3:
+            if event.state & gtk.gdk.MOD1_MASK:
+                self._window.previous_page()
+            elif event.state & gtk.gdk.SHIFT_MASK:
+                self._window.previous_page_fast_forward()
 
     def mouse_move_event(self, widget, event):
         """Handle mouse pointer movement events."""
@@ -561,7 +581,7 @@ class EventHandler:
 
         if (not self._scroll_protection
             or self._extra_scroll_events >= prefs['number of key presses before page turn'] - 1
-            or not self._window.is_scrollable_vertically()):
+            or not self._window.is_scrollable()):
 
             self._extra_scroll_events = 0
             self._window.next_page()
@@ -587,7 +607,7 @@ class EventHandler:
 
         if (not self._scroll_protection
             or self._extra_scroll_events <= -prefs['number of key presses before page turn'] + 1
-            or not self._window.is_scrollable_vertically()):
+            or not self._window.is_scrollable()):
 
             self._extra_scroll_events = 0
             self._window.previous_page()
@@ -601,6 +621,14 @@ class EventHandler:
             # This path should not be reached.
             assert False, "Programmer is moron, incorrect assertion."
 
+    def _execute_command(self, cmdindex):
+        """ Execute an external command. cmdindex should be an integer from 0 to 9,
+        representing the command that should be executed. """
+        manager = openwith.OpenWithManager()
+        commands = [cmd for cmd in manager.get_commands() if not cmd.is_separator()]
+        if len(commands) > cmdindex:
+            commands[cmdindex].execute(self._window)
+
 
 def _get_latest_event_of_same_type(event):
     """Return the latest event in the event queue that is of the same type
