@@ -20,6 +20,7 @@ from mcomix import callback
 from mcomix import log
 from mcomix import last_read_page
 from mcomix import message_dialog
+from mcomix.library import backend
 
 class FileHandler(object):
 
@@ -60,7 +61,7 @@ class FileHandler(object):
         #: Provides a list of available files/archives in the open directory.
         self._file_provider = None
         #: Keeps track of the last read page in archives
-        self.last_read_page = last_read_page.LastReadPage(constants.LASTPAGE_DATABASE_PATH)
+        self.last_read_page = last_read_page.LastReadPage(backend.LibraryBackend())
         #: Regexp used for determining which archive files are images.
         self._image_re = constants.SUPPORTED_IMAGE_REGEX
         #: Regexp used for determining which archive files are comment files.
@@ -69,14 +70,17 @@ class FileHandler(object):
         #: Forces call to window.draw_image (if loading is delayed by user interaction)
         self._must_call_draw = False
 
-        self.last_read_page.set_enabled(
-            prefs['store recent file info'] == constants.STORE_LAST_PATH_AND_PAGE)
+        self.last_read_page.set_enabled(bool(prefs['store recent file info']))
 
     def refresh_file(self, *args, **kwargs):
         """ Closes the current file(s)/archive and reloads them. """
         if self.file_loaded:
             current_file = os.path.abspath(self._window.imagehandler.get_real_path())
-            self.open_file(current_file, keep_fileprovider=True)
+            if self.archive_type is not None:
+                start_page = self._window.imagehandler.get_current_page()
+            else:
+                start_page = 0
+            self.open_file(current_file, start_page, keep_fileprovider=True)
 
     def open_file(self, path, start_page=0, keep_fileprovider=False):
         """Open the file pointed to by <path>.
@@ -196,8 +200,15 @@ class FileHandler(object):
             self._must_call_draw = False
             self._window.draw_image()
 
+        self.file_opened()
         return result
 
+    @callback.Callback
+    def file_opened(self):
+        """ Called when a new set of files has successfully been opened. """
+        pass
+
+    @callback.Callback
     def close_file(self, *args):
         """Run tasks for "closing" the currently opened file(s)."""
         self.update_last_read_page()
@@ -304,7 +315,7 @@ class FileHandler(object):
                 # Remove MacOS meta files from image list
                 and not u'__MACOSX' in os.path.normpath(image).split(os.sep)]
 
-            tools.alphanumeric_sort(archive_images)
+            archive_images = self._sort_archive_images(archive_images)
             image_files = [ os.path.join(self._tmp_dir, f)
                             for f in archive_images ]
 
@@ -399,6 +410,24 @@ class FileHandler(object):
             # No condition was returned from the Extractor, i.e. invalid archive.
             return [], 0
 
+    def _sort_archive_images(self, filelist):
+        """ Sorts the image list passed in C{filelist} based on the sorting
+        preference option, and returns the newly sorted list. """
+        filelist = list(filelist)  # Create a copy of the list
+
+        if prefs['sort archive by'] == constants.SORT_NAME:
+            tools.alphanumeric_sort(filelist)
+        elif prefs['sort archive by'] == constants.SORT_NAME_LITERAL:
+            filelist.sort()
+        else:
+            # No sorting
+            pass
+
+        if prefs['sort archive order'] == constants.SORT_DESCENDING:
+            filelist.reverse()
+
+        return filelist
+
     def _save_state(self):
         """ Saves the FileHandler's internal state and returns it as dict object,
         which can be loaded by L{_restore_state}. """
@@ -516,7 +545,6 @@ class FileHandler(object):
             self._condition.release()
 
         self.update_last_read_page()
-        self.last_read_page.cleanup()
 
     def get_number_of_comments(self):
         """Return the number of comments in the current archive."""
@@ -690,11 +718,11 @@ class FileHandler(object):
         """
         pass
 
-    def _extracted_file(self, name):
+    def _extracted_file(self, extractor, name):
         """ Called when the extractor finishes extracting the file at
         <name>. This name is relative to the temporary directory
         the files were extracted to. """
-        filepath = os.path.join(self._tmp_dir, name)
+        filepath = os.path.join(extractor.get_directory(), name)
         self.file_available([filepath])
 
     def _wait_on_page(self, page):
@@ -781,13 +809,16 @@ class FileHandler(object):
 
         archive_path = self.get_path_to_base()
         page = self._window.imagehandler.get_current_page()
-        is_last_page = self._window.imagehandler.is_last_page()
-        # Do not store first and last page (last page means archive was finished,
-        # first page is default behaviour and would waste space unnecessarily)
-        if page == 1 or is_last_page:
-            self.last_read_page.clear_page(archive_path)
-        else:
-            self.last_read_page.set_page(archive_path, page)
+        # Do not store first page (first page is default
+        # behaviour and would waste space unnecessarily)
+        try:
+            if page == 1:
+                self.last_read_page.clear_page(archive_path)
+            else:
+                self.last_read_page.set_page(archive_path, page)
+        except ValueError:
+            # The book no longer exists in the library and has been deleted
+            pass
 
 
 # vim: expandtab:sw=4:ts=4
