@@ -9,6 +9,8 @@ from mcomix import thumbnail_tools
 from mcomix import log
 from mcomix import callback
 from mcomix.library import backend_types
+# Only for importing legacy data from last-read module
+from mcomix import last_read_page
 
 try:
     from sqlite3 import dbapi2
@@ -20,6 +22,10 @@ except ImportError:
         dbapi2 = None
 
 
+#: Identifies the 'Recent' collection that stores recently read books.
+COLLECTION_RECENT = -2
+
+
 class _LibraryBackend:
 
     """The LibraryBackend handles the storing and retrieval of library
@@ -28,7 +34,7 @@ class _LibraryBackend:
 
     #: Current version of the library database structure.
     # See method _upgrade_database() for changes between versions.
-    DB_VERSION = 4
+    DB_VERSION = 5
 
     def __init__(self):
 
@@ -40,14 +46,20 @@ class _LibraryBackend:
                 return row[0]
             return row
 
-        self._con = dbapi2.connect(constants.LIBRARY_DATABASE_PATH,
-            check_same_thread=False, isolation_level=None)
-        self._con.row_factory = row_factory
+        if dbapi2 is not None:
+            self._con = dbapi2.connect(constants.LIBRARY_DATABASE_PATH,
+                check_same_thread=False, isolation_level=None)
+            self._con.row_factory = row_factory
+            self.enabled = True
 
-        self.watchlist = backend_types._WatchList(self)
+            self.watchlist = backend_types._WatchList(self)
 
-        version = self._library_version()
-        self._upgrade_database(version, _LibraryBackend.DB_VERSION)
+            version = self._library_version()
+            self._upgrade_database(version, _LibraryBackend.DB_VERSION)
+        else:
+            self._con = None
+            self.watchlist = None
+            self.enabled = False
 
     def get_books_in_collection(self, collection=None, filter_string=None):
         """Return a sequence with all the books in <collection>, or *ALL*
@@ -77,6 +89,40 @@ class _LibraryBackend:
                 books.extend(cur.fetchall())
             return books
 
+    def get_book_by_path(self, path):
+        """ Retrieves a book from the library, specified by C{path}.
+        If the book doesn't exist, None is returned. Otherwise, a
+        L{backend_types._Book} instance is returned. """
+
+        path = os.path.abspath(path)
+
+        cur = self.execute('''select id, name, path, pages, format,
+                                     size, added
+                              from book where path = ?''', (path,))
+        book = cur.fetchone()
+        cur.close()
+
+        if book:
+            return backend_types._Book(*book)
+        else:
+            return None
+
+    def get_book_by_id(self, id):
+        """ Retrieves a book from the library, specified by C{id}.
+        If the book doesn't exist, C{None} is returned. Otherwise, a
+        L{backend_types._Book} instance is returned. """
+
+        cur = self.execute('''select id, name, path, pages, format,
+                                     size, added
+                              from book where id = ?''', (id,))
+        book = cur.fetchone()
+        cur.close()
+
+        if book:
+            return backend_types._Book(*book)
+        else:
+            return None
+
     def get_book_cover(self, book):
         """Return a pixbuf with a thumbnail of the cover of <book>, or
         None if the cover can not be fetched.
@@ -239,6 +285,11 @@ class _LibraryBackend:
             else:
                 return None
 
+    def get_recent_collection(self):
+        """ Returns the "Recent" collection, especially created for
+        storing recently opened files. """
+        return self.get_collection_by_id(COLLECTION_RECENT)
+
     def get_supercollection(self, collection):
         """Return the supercollection of <collection>."""
         cur = self._con.execute('''select supercollection from Collection
@@ -303,8 +354,17 @@ class _LibraryBackend:
         if the collection was successfully added.
         """
         try:
-            self._con.execute('''insert into Collection
-                (name) values (?)''', (name,))
+            # The Recent pseudo collection initializes the lowest rowid
+            # with -2, meaning that instead of starting from 1,
+            # auto-incremental will start from -1. Avoid this.
+            cur = self._con.execute('''select max(id) from collection''')
+            maxid = cur.fetchone()
+            if maxid is not None and maxid < 1:
+                self._con.execute('''insert into collection
+                    (id, name) values (?, ?)''', (1, name))
+            else:
+                self._con.execute('''insert into Collection
+                    (name) values (?)''', (name,))
             return True
         except dbapi2.Error:
             log.error( _('! Could not add collection "%s"'), name )
@@ -410,10 +470,23 @@ class _LibraryBackend:
         connection. """
         return self._con.execute(*args)
 
+    def begin_transaction(self):
+        """ Normally, the connection is in auto-commit mode. Calling
+        this method will switch to transactional mode, automatically
+        starting a transaction when a DML statement is used. """
+        self._con.isolation_level = 'IMMEDIATE'
+
+    def end_transaction(self):
+        """ Commits any changes to the database and switches back
+        to auto-commit mode. """
+        self._con.commit()
+        self._con.isolation_level = None
+
     def close(self):
         """Commit changes and close cleanly."""
-        self._con.commit()
-        self._con.close()
+        if self._con is not None:
+            self._con.commit()
+            self._con.close()
 
         global _backend
         _backend = None
@@ -460,6 +533,7 @@ class _LibraryBackend:
         self._create_table_contain()
         self._create_table_info()
         self._create_table_watchlist()
+        self._create_table_recent()
 
     def _upgrade_database(self, from_version, to_version):
         """ Performs sequential upgrades to the database, bringing
@@ -505,6 +579,13 @@ class _LibraryBackend:
                     select path, collection, 0 from watchlist_old''')
                 self._con.execute('''drop table watchlist_old''')
 
+            if 4 in upgrades:
+                # Added table 'recent' to store recently viewed book information and
+                # create a collection (-2, Recent)
+                self._create_table_recent()
+                lastread = last_read_page.LastReadPage(self)
+                lastread.migrate_database_to_library(COLLECTION_RECENT)
+
             self._con.execute('''update info set value = ? where key = 'version' ''',
                               (str(_LibraryBackend.DB_VERSION),))
 
@@ -544,6 +625,14 @@ class _LibraryBackend:
             collection integer references collection (id) on delete set null,
             recursive boolean not null)''')
 
+    def _create_table_recent(self):
+        self._con.execute('''create table if not exists recent (
+            book integer primary key,
+            page integer,
+            time_set datetime)''')
+        self._con.execute('''insert or ignore into collection (id, name)
+            values (?, ?)''', (COLLECTION_RECENT, _('Recent')))
+
 
 _backend = None
 
