@@ -2,6 +2,7 @@
 
 import os
 import threading
+import datetime
 
 from mcomix import callback
 from mcomix import archive_tools
@@ -36,6 +37,73 @@ class _Book(_BackendObject):
         self.size = size
         self.added = added
 
+    def get_collections(self):
+        """ Gets a list of collections this book is part of. If it
+        belongs to no collections, [DefaultCollection] is returned. """
+        cursor = self.get_backend().execute(
+            '''SELECT id, name, supercollection FROM collection
+               JOIN contain on contain.collection = collection.id
+               WHERE contain.book = ?''', (self.id,))
+        rows = cursor.fetchall()
+        if rows:
+            return [_Collection(*row) for row in rows]
+        else:
+            return [DefaultCollection]
+
+    def get_last_read_page(self):
+        """ Gets the page of this book that was last read when the book was
+        closed. Returns C{None} if no such page exists. """
+        cursor = self.get_backend().execute(
+            '''SELECT page FROM recent WHERE book = ?''', (self.id,))
+        row = cursor.fetchone()
+        cursor.close()
+        if row:
+            return row
+        else:
+            return None
+
+    def get_last_read_date(self):
+        """ Gets the datetime the book was most recently read. Returns
+        C{None} if no information was set, or a datetime object otherwise. """
+        cursor = self.get_backend().execute(
+            """SELECT time_set FROM recent WHERE book = ?""", (self.id,))
+        date = cursor.fetchone()
+        cursor.close()
+
+        if date:
+            try:
+                return datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S.%f')
+            except ValueError:
+                # Certain operating systems do not store fractions
+                return datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
+        else:
+            return None
+
+    def set_last_read_page(self, page, time=None):
+        """ Sets the page that was last read when the book was closed.
+        Passing C{None} as argument clears the recent information.
+
+        @param page: Page number, starting from 1 (page 1 throws ValueError)
+        @param time: Time of reading. If None, current time is used. """
+
+        if page is not None and page < 1:
+            # Avoid wasting memory by creating a recently viewed entry when
+            # an archive was opened on page 1.
+            raise ValueError('Invalid page (must start from 1)')
+
+        # Remove any old recent row for this book
+        cursor = self.get_backend().execute(
+            '''DELETE FROM recent WHERE book = ?''', (self.id,))
+        # If a new page was passed, set it as recently read
+        if page is not None:
+            if not time:
+                time = datetime.datetime.now()
+            cursor.execute('''INSERT INTO recent (book, page, time_set)
+                              VALUES (?, ?, ?)''',
+                           (self.id, page, time))
+
+        cursor.close()
+
 
 class _Collection(_BackendObject):
     """ Library collection instance.
@@ -55,6 +123,8 @@ class _Collection(_BackendObject):
     def __eq__(self, other):
         if isinstance(other, _Collection):
             return self.id == other.id
+        elif isinstance(other, (int, long)):
+            return self.id == other
         else:
             return False
 
@@ -237,7 +307,9 @@ class _WatchList(object):
 
     def _scan_for_new_files_thread(self):
         """ Executes the actual scanning operation in a new thread. """
-        existing_books = [book.path for book in DefaultCollection.get_books()]
+        existing_books = [book.path for book in DefaultCollection.get_books()
+                          # Also add book if it was only found in Recent collection
+                          if book.get_collections() != [-2]]
         for entry in self.get_watchlist():
             new_files = entry.get_new_files(existing_books)
             self.new_files_found(new_files, entry)
