@@ -4,8 +4,9 @@ import os
 import urllib
 import gtk
 import gobject
-import Image
-import ImageDraw
+import PIL.Image as Image
+import PIL.ImageDraw as ImageDraw
+import pkg_resources
 
 from mcomix.preferences import prefs
 from mcomix import thumbnail_view
@@ -248,7 +249,11 @@ class _BookArea(gtk.ScrolledWindow):
 
         if (collection == self._library.collection_area.get_current_collection() or
             self._library.collection_area.get_current_collection() == _COLLECTION_ALL):
-            self.add_books([book])
+
+            # If the current view is filtered, only draw new books that match the filter
+            if not (self._library.filter_string and 
+                    self._library.filter_string.lower() not in book.name.lower()):
+                self.add_books([book])
 
     def remove_book_at_path(self, path):
         """Remove the book at <path> from the ListStore (and thus from
@@ -393,7 +398,7 @@ class _BookArea(gtk.ScrolledWindow):
     def _get_pixbuf(self, path, model, model_path):
         """ Get or create the thumbnail for the selected book at <path>. """
         if self._cache.exists(path):
-            return self._cache.get(path)
+            pixbuf = self._cache.get(path)
         else:
             pixbuf = self._library.backend.get_book_thumbnail(path) or constants.MISSING_IMAGE_ICON
             # The ratio (0.67) is just above the normal aspect ratio for books.
@@ -403,8 +408,31 @@ class _BookArea(gtk.ScrolledWindow):
             pixbuf = image_tools.add_border(pixbuf, 1, 0xFFFFFFFF)
             self._cache.add(path, pixbuf)
 
+        # Display indicator of having finished reading the book.
+        # This information isn't cached in the pixbuf cache, as it changes frequently.
+
+        # Anything smaller than 50px means that the status icon will not fit
+        if prefs['library cover size'] < 50:
             return pixbuf
 
+        book = self._library.backend.get_book_by_path(path)
+        last_read_page = book.get_last_read_page()
+
+        if last_read_page is None or last_read_page != book.pages:
+            return pixbuf
+        if last_read_page == book.pages:
+            book_pixbuf = self.render_icon(gtk.STOCK_APPLY, gtk.ICON_SIZE_LARGE_TOOLBAR)
+
+        # Composite icon on the lower right corner of the book cover pixbuf.
+        translation_x = pixbuf.get_width() - book_pixbuf.get_width() - 1
+        translation_y = pixbuf.get_height() - book_pixbuf.get_height() - 1
+        book_pixbuf.composite(pixbuf, translation_x, translation_y,
+                              book_pixbuf.get_width(), book_pixbuf.get_height(),
+                              translation_x, translation_y,
+                              1.0, 1.0, gtk.gdk.INTERP_NEAREST, 0xFF)
+
+        return pixbuf
+
     def _get_empty_thumbnail(self):
         """ Create an empty filler pixmap. """
         pixbuf = gtk.gdk.Pixbuf(colorspace=gtk.gdk.COLORSPACE_RGB,
@@ -440,10 +468,13 @@ class _BookArea(gtk.ScrolledWindow):
         if collection == _COLLECTION_ALL:
             return
         selected = self._iconview.get_selected_items()
+        self._library.backend.begin_transaction()
         for path in selected:
             book = self.get_book_at_path(path)
             self._library.backend.remove_book_from_collection(book, collection)
             self.remove_book_at_path(path)
+        self._library.backend.end_transaction()
+
         coll_name = self._library.backend.get_collection_name(collection)
         message = i18n.get_translation().ungettext(
                 "Removed %(num)d book from '%(collection)s'.",
@@ -458,12 +489,15 @@ class _BookArea(gtk.ScrolledWindow):
         """
 
         selected = self._iconview.get_selected_items()
+        self._library.backend.begin_transaction()
 
         for path in selected:
             book = self.get_book_at_path(path)
             self._library.backend.remove_book(book)
             self.remove_book_at_path(path)
 
+        self._library.backend.end_transaction()
+
         msg = i18n.get_translation().ungettext(
             'Removed %d book from the library.',
             'Removed %d books from the library.',
