@@ -360,9 +360,12 @@ class _PreferencesDialog(gtk.Dialog):
 
         page.new_section(_('File order'))
 
-        label = gtk.Label(_('Order files by:'))
+        label = gtk.Label(_('Sort files and directories by:'))
         page.add_row(label, self._create_sort_by_control())
 
+        label = gtk.Label(_('Sort archives by:'))
+        page.add_row(label, self._create_archive_sort_by_control())
+
         page.new_section(_('Cache'))
 
         create_thumbs_button = gtk.CheckButton(
@@ -518,7 +521,7 @@ class _PreferencesDialog(gtk.Dialog):
                 self._window.change_zoom_mode()
 
     def _create_sort_by_control(self):
-        """ Creates the ComboBox control for selecting archive sort by options. """
+        """ Creates the ComboBox control for selecting file sort by options. """
         sortkey_items = (
                 (_('No sorting'), 0),
                 (_('File name'), constants.SORT_NAME),
@@ -565,18 +568,66 @@ class _PreferencesDialog(gtk.Dialog):
 
             self._window.filehandler.refresh_file()
 
+    def _create_archive_sort_by_control(self):
+        """ Creates the ComboBox control for selecting archive sort by options. """
+        sortkey_items = (
+                (_('No sorting'), 0),
+                (_('Natural order'), constants.SORT_NAME),
+                (_('Literal order'), constants.SORT_NAME_LITERAL))
+
+        sortkey_box = self._create_combobox(sortkey_items, prefs['sort archive by'],
+            self._sort_archive_by_changed_cb)
+
+        sortorder_items = (
+                (_('Ascending'), constants.SORT_ASCENDING),
+                (_('Descending'), constants.SORT_DESCENDING))
+
+        sortorder_box = self._create_combobox(sortorder_items,
+                prefs['sort archive order'],
+                self._sort_archive_order_changed_cb)
+
+        box = gtk.HBox()
+        box.pack_start(sortkey_box)
+        box.pack_start(sortorder_box)
+
+        label = _("Files within archives will be sorted according to the order specified here. "
+                  "Natural order will sort numbered files based on their natural order, "
+                  "i.e. 1, 2, ..., 10, while literal order uses standard C sorting, "
+                  "i.e. 1, 2, 34, 5.")
+        sortkey_box.set_tooltip_text(label)
+        sortorder_box.set_tooltip_text(label)
+
+        return box
+
+    def _sort_archive_by_changed_cb(self, combobox, *args):
+        """ Called when a new option was selected for the virtual double page option. """
+        iter = combobox.get_active_iter()
+        if combobox.get_model().iter_is_valid(iter):
+            value = combobox.get_model().get_value(iter, 1)
+            prefs['sort archive by'] = value
+
+            self._window.filehandler.refresh_file()
+
+    def _sort_archive_order_changed_cb(self, combobox, *args):
+        """ Called when sort order changes (ascending or descending) """
+        iter = combobox.get_active_iter()
+        if combobox.get_model().iter_is_valid(iter):
+            value = combobox.get_model().get_value(iter, 1)
+            prefs['sort archive order'] = value
+
+            self._window.filehandler.refresh_file()
+
     def _create_store_recent_combobox(self):
         """ Creates the combobox for "Store recently opened files". """
         items = (
-                (_('Never'), 0),
-                (_('Only file names'), constants.STORE_LAST_PATH),
-                (_('File names and last read page'), constants.STORE_LAST_PATH_AND_PAGE))
+                (_('Never'), False),
+                (_('Always'), True))
 
-        # Map legacy true/false values:
-        if prefs['store recent file info'] is True:
-            selection = constants.STORE_LAST_PATH
-        elif prefs['store recent file info'] is False:
-            selection = 0
+        # Map legacy 0/1/2 values:
+        if prefs['store recent file info'] == 0:
+            selection = False
+        elif prefs['store recent file info'] in (1, 2):
+            selection = True
         else:
             selection = prefs['store recent file info']
 
@@ -592,11 +643,10 @@ class _PreferencesDialog(gtk.Dialog):
         value = combobox.get_model().get_value(iter, 1)
         last_value = prefs['store recent file info']
         prefs['store recent file info'] = value
-        self._window.filehandler.last_read_page.set_enabled(
-            value == constants.STORE_LAST_PATH_AND_PAGE)
+        self._window.filehandler.last_read_page.set_enabled(value)
 
         # If "Never" was selected, ask to purge recent files.
-        if (last_value > 0 and value == 0
+        if (bool(last_value) is True and value is False
             and (self._window.uimanager.recent.count() > 0
                  or self._window.filehandler.last_read_page.count() > 0)):
 
