@@ -76,6 +76,9 @@ def parse_arguments(argv):
             choices=('all', 'warn', 'error'), default='warn',
             metavar='[ all | warn | error ]',
             help=_('Sets the desired output log level.'))
+    # This supresses an error when MComix is used with cProfile
+    debugopts.add_option('-o', dest='output', action='store',
+            default='', help=optparse.SUPPRESS_HELP)
     parser.add_option_group(debugopts)
 
     opts, args = parser.parse_args(argv)
@@ -138,8 +141,8 @@ except ImportError:
 
 # Check PIL library
 try:
-    import Image
-    assert Image.VERSION >= '1.1.5'
+    import PIL.Image
+    assert PIL.Image.VERSION >= '1.1.5'
 
 except AssertionError:
     print_( _("You don't have the required version of the Python Imaging"), end=' ')
@@ -186,6 +189,11 @@ def run():
         open_page = preferences.prefs['page of last file']
 
     log.setLevel(opts.loglevel)
+
+    # Some languages require a RTL layout
+    if preferences.prefs['language'] in ('he', 'fa'):
+        gtk.widget_set_default_direction(gtk.TEXT_DIR_RTL)
+
     window = main.MainWindow(fullscreen = opts.fullscreen, is_slideshow = opts.slideshow,
             show_library = opts.library, manga_mode = opts.manga,
             double_page = opts.doublepage, zoom_mode = opts.zoommode,
