@@ -5,6 +5,7 @@ import os.path
 import gtk
 
 from mcomix import bookmark_menu
+from mcomix import openwith_menu
 from mcomix import edit_dialog
 from mcomix import enhance_dialog
 from mcomix import preferences_dialog
@@ -13,6 +14,7 @@ from mcomix import dialog_handler
 from mcomix import constants
 from mcomix import status
 from mcomix import file_chooser_main_dialog
+from mcomix.preferences import prefs
 from mcomix.library import main_dialog as library_main_dialog
 
 class MainUI(gtk.UIManager):
@@ -89,6 +91,8 @@ class MainUI(gtk.UIManager):
             ('menu_bookmarks', None, _('_Bookmarks')),
             ('menu_toolbars', None, _('T_oolbars')),
             ('menu_edit', None, _('_Edit')),
+            ('menu_open_with', gtk.STOCK_OPEN, _('Open _with'), ''),
+            ('menu_open_with_popup', gtk.STOCK_OPEN, _('Open _with'), ''),
             ('menu_file', None, _('_File')),
             ('menu_view', None, _('_View')),
             ('menu_view_popup', 'comix-image', _('_View')),
@@ -97,6 +101,9 @@ class MainUI(gtk.UIManager):
             ('menu_tools', None, _('_Tools')),
             ('menu_help', None, _('_Help')),
             ('menu_transform', 'mcomix-transform', _('_Transform image')),
+            ('menu_autorotate', None, _('_Auto-rotate image')),
+            ('menu_autorotate_width', None, _('...when width exceeds height')),
+            ('menu_autorotate_height', None, _('...when height exceeds width')),
             ('expander', None, None, None, None, None)])
 
         self._actiongroup.add_toggle_actions([
@@ -146,6 +153,20 @@ class MainUI(gtk.UIManager):
                 'a', _('Manual zoom mode'), constants.ZOOM_MODE_MANUAL)],
             3, window.change_zoom_mode)
 
+        # Automatically rotate image if width>height or height>width
+        self._actiongroup.add_radio_actions([
+            ('no_autorotation', None, _('Never'),
+             None, None, constants.AUTOROTATE_NEVER),
+            ('rotate_90_width', 'mcomix-rotate-90', _('_Rotate 90 degrees CW'),
+             None, None, constants.AUTOROTATE_WIDTH_90),
+            ('rotate_270_width', 'mcomix-rotate-270', _('Rotat_e 90 degrees CCW'),
+             None, None, constants.AUTOROTATE_WIDTH_270),
+            ('rotate_90_height', 'mcomix-rotate-90', _('_Rotate 90 degrees CW'),
+             None, None, constants.AUTOROTATE_HEIGHT_90),
+            ('rotate_270_height', 'mcomix-rotate-270', _('Rotat_e 90 degrees CCW'),
+             None, None, constants.AUTOROTATE_HEIGHT_270)],
+            prefs['auto rotate depending on size'], window.change_autorotation)
+
         self._actiongroup.add_actions([
             ('about', gtk.STOCK_ABOUT, _('_About'),
              None, None, dialog_handler.open_dialog)], (window, 'about-dialog'))
@@ -211,6 +232,8 @@ class MainUI(gtk.UIManager):
                     <menuitem action="refresh_archive" />
                     <menuitem action="properties" />
                     <separator />
+                    <menu action="menu_open_with"></menu>
+                    <separator />
                     <menuitem action="delete" />
                     <separator />
                     <menuitem action="minimize" />
@@ -280,6 +303,20 @@ class MainUI(gtk.UIManager):
                         <menuitem action="rotate_270" />
                         <menuitem action="rotate_180" />
                         <separator />
+                        <menu action="menu_autorotate">
+                            <menuitem action="no_autorotation" />
+                            <separator />
+                            <menuitem action="menu_autorotate_height" />
+                            <separator />
+                            <menuitem action="rotate_90_height" />
+                            <menuitem action="rotate_270_height" />
+                            <separator />
+                            <menuitem action="menu_autorotate_width" />
+                            <separator />
+                            <menuitem action="rotate_90_width" />
+                            <menuitem action="rotate_270_width" />
+                        </menu>
+                        <separator />
                         <menuitem action="flip_horiz" />
                         <menuitem action="flip_vert" />
                         <separator />
@@ -346,9 +383,10 @@ class MainUI(gtk.UIManager):
                 <menu action="menu_recent" />
                 <menuitem action="library" />
                 <separator />
+                <menu action="menu_open_with_popup"></menu>
+                <separator />
                 <menuitem action="preferences" />
                 <separator />
-                <menuitem action="minimize" />
                 <menuitem action="close" />
                 <menuitem action="quit" />
             </popup>
@@ -374,6 +412,13 @@ class MainUI(gtk.UIManager):
         self.get_widget('/Popup/menu_recent').set_submenu(self.recentPopup)
         self.get_widget('/Popup/menu_recent').show()
 
+        openwith = openwith_menu.OpenWithMenu(self, window)
+        self.get_widget('/Menu/menu_file/menu_open_with').set_submenu(openwith)
+        self.get_widget('/Menu/menu_file/menu_open_with').show()
+        openwith = openwith_menu.OpenWithMenu(self, window)
+        self.get_widget('/Popup/menu_open_with_popup').set_submenu(openwith)
+        self.get_widget('/Popup/menu_open_with_popup').show()
+
         # Load keyboard accelerator map
         if os.path.isfile(constants.KEYBINDINGS_PATH):
             gtk.accel_map_load(constants.KEYBINDINGS_PATH)
